# # # # # # # # # # # # # # # # # # # # # # #

# INSTALL.S - CIEX Base Installation Procedure
# CIEX (C) 1993, 1994 New Line Software, Inc.
# Script Version 1.2

# Usage: source install.s [ inline ]

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Install the CIEX base distribution files.

# # # # # # # # # # # # # # # # # # # # # # #

set -clear

# These parameters may be changed to relocate the demo program.

set $target_drive     = "C"           # Name of the target drive
set $target_directory = "\CIEX"       # Name of the target directory.
set $pif_directory    = "\CIEX\PIF"   # Name Windows PIF directory.

# These parameters affect the look of the display.

set $title_color      = green
set $box_color        = cyan
set $text_color       = white
set $hilite_color     = red

# The "inline" option, if specified, provides special facility to
# continue with the demo, and is used when the "install.s" script is 
# sourced by the "demo.s" script: 

if $arg:1^value eq "inline"
   set $inline = true
else
   set $inline = false
endif

callpoint: top

# Display a screen.

screen -default
screen -clear
screen -buff on
box -style french

# Draw main window.

screen -fore $box_color
box 1 1 78 24

# Main window title.

#screen -fore $box_color
#line -invert

# Display instructions.

screen -fore $box_color
move  1 1  ; echo -center "  THE CIEX INTERPRETER  "
move -down ; echo -center "Copyright 1993, 1994 New Line Software, Inc."
move -down ; echo -center "WOULD YOU LIKE TO INSTALL CIEX?"

# Instructions are:

screen -fore $text_color
move 6 4
echoblock -screen

The CIEX interpreter works best when executed from the hard drive 
of a model 386 or 486 PC. The program will execute from floppy disks 
and earlier PC models, but performance will be MUCH SLOWER. We suggest
you select the "F1" key (below) to install the interpreter on your 
hard disk now. (Installation requires 500 K-Bytes of disk space).

endblock

# Draw menu box.

screen -fore $hilite_color
box 5 11 70 11
screen -fore $title_color

# The menu selections are as follows:
# Note that the particular menu display depends upon the presence of the
# "-inline" command line qualifier.

move 13 11

if $inline eq true
   echo -center '  SELECT FUNCTION KEY: F1, F2, F3, OR F4  '
   move -down 2; echo -screen '   F1   ' ; line -invert 8
   move -down 2; echo -screen '   F2   ' ; line -invert 8
   move -down 2; echo -screen '   F3   ' ; line -invert 8
   move -down 2; echo -screen '   F4   ' ; line -invert 8
else
   echo -center '  SELECT A FUNCTION KEY: F1, F2, OR F3  '
   move -down 3; echo -screen '   F1   ' ; line -invert 8
   move -down 2; echo -screen '   F2   ' ; line -invert 8
   move -down 2; echo -screen '   F3   ' ; line -invert 8
endif

move 27 11
screen -fore $text_color

if $inline eq true
  
   # Provide menu option to continue with demo program.

   move -down 2
   echo -screen "Install programs in $target_drive~:$target_directory"
   
   move -down 2
   echo -screen "Change target disk drive for installation."
   
   move -down 2
   echo -screen "Exit program now with no action taken."
   
   move -down 2
   echo -screen "Do not install. Continue with demo."

else

   # This is a stand-alone installation.

   move -down 3
   echo -screen "Install programs in $target_drive~:$target_directory"
   
   move -down 2
   echo -screen "Change target disk drive for installation."
   
   move -down 2
   echo -screen "Exit program now with no action taken."

endif

# Display the NLS logo.

call display_logo 23 22

# Display the screen.

screen -flush
screen -buff off

move -home

# Loop until a valid key is entered.

loop

   # Wait for a key. The program stops.

   read -key $key

   # User entered a key. Decode the key as follows:

   if      -test $key eq 'f1' ; break       # Perform the installation.
   else if -test $key eq 'f2' 

      # Change the target drive designation. When finished, go to the
      # top to redisplay the screen.

      move 13 11
      call set_drive_name
      call -goto top

   else if -test $key eq 'f3' ; return -all # Return to DOS.
   else if -test $key eq 'f4' and $inline eq true

      # User wants to bypass the installation.
      # Before proceeding to the demo program, provide a screen
      # that reminds the user that CIEX works best when installed on a
      # hard disk drive.

      scr -clear
      screen -fore $box_color
      box -style french
      box 10 6

      scr -fore $title_color
      move 1 6;  echo -center '   CIEX (C) 1993, INSTALLATION   '
      move 1 18; echo -center '   [ ANY KEY TO CONTINUE ]   '

      move 12 7
      screen -fore $text_color

      echoblock -center

      Okay. Bypassing installation....

      PLEASE NOTE!
      CIEX runs much faster when installed on a hard disk!

      To permanently disable this installation screen, 
      rename (or remove) the file: INSTALL.S 
      in the local directory.

      endblock

      # Wait for a keystroke before continuing with the demo.

      move -home
      read -key $key

      # Begin the demo program.

      screen -clear
      return

   endif

   # Unknown key. Go to the top of the loop.

endloop

# The user selected F1. Perform the installation as follows:

cls
set $target_path = $target_drive
set $target_path & ":"
set $target_path & $target_directory

if -test $target_path is not directory

   # Create the target directory:

   echo "Creating $target_path ..."
   call mkdir $target_path

endif

# Main files are as follows:

set -cmd $file_list = echoblock -wrap 1

   DEMO.BAT
   CIEX.EXE
   DEMO.S
   INTRO.S
   EVAL.S
   ADDR.S
   CALENDAR.S
   REACTOR.S
   README.TXT
   ORDERFRM.TXT

endblock

loop for $file = $file_list

   if $file is file
      echo "Copying $file "
      copy $file $target_path
   endif

endloop

# Copy the following files to the target PIF directory.

set $pif_path = $target_drive
set $pif_path & ":"
set $pif_path & $pif_directory

if -test $pif_path is not directory

   # Create the PIF directory:

   echo "Creating $pif_path ..."
   call mkdir $pif_path

endif

set -cmd $file_list = echoblock -wrap 1

   EVAL.PIF
   ADDR.PIF
   CALENDAR.PIF

endblock

loop for $file = $file_list

   if $file is file
      echo "Copying $file "
      copy $file $pif_path
   endif

endloop

# Display a sign-off message.

scroll -up 14
screen -fore $box_color
scr -buffer on
box  1 14 42 9
screen -fore $text_color

move 9 14   ; echo -screen " INSTALLATION IS COMPLETE. "
move 3 16   ; echo -screen " Working directory is now: $target_path"
move -down  ; echo -screen " Execute DEMO to continue."

call display_logo 4 19

set $target_drive & ':'
cd $target_path
exec $target_drive
scr -flush
scr -buffer off

# Return to DOS.

return -all

# # # # # # # # # # # # # # #

callpoint: get_drive_name

# Get the name of this drive.

# # # # # # # # # # # # # # #

set -inline $a = cd
set -list $a = $a by ':'
echo $a:1

# # # # # # # # # # # # # # #

callpoint: set_drive_name

# Set a new operant drive.

# # # # # # # # # # # # # # #

# Prompt for a drive letter.

scr -clear
screen -fore $box_color
box -style french
box 10 4

move 1 7
scr -fore $title_color
echo -center '[  ENTER - TARGET DISK DRIVE LETTER (A-Z)?  ]'

move 17 8
scr -fore $text_color
echoblock -screen

Enter a letter corresponding to a valid disk 
drive, in the range of A to Z. The target drive 
for the installation will be set to this letter. 
No installation will actually be performed until 
confirmed by the operator from the Main menu. 

endblock

# Wait for a key, then convert key to upper case.

read -key $drive_name
set -inlin $drive_name = echo -upper $drive_name
move 1 16

# Verify that selected drive is online, and report.

scr -fore $text_color

if $drive_name eq ESC or $drive_name eq ENTER

   # Return with no action.

   return

else if $drive_name llt 'A' ~
   or   $drive_name lgt 'Z' ~
   or   $drive_name^size ne 1

   # The drive name is out of range.

   echo -center "!? - Illegal disk drive name."

else if -cmd

   # Handle exception if drive name is not available.

   echo > $drive_name~:\nul   
   echo -center "Okay - using disk drive:  $drive_name"
   set $target_drive = $drive_name
else

   # Exception encountered.

   echo -center "!? - Invalid disk drive: $drive_name"

endif

scr -fore $title_color
move -down 2 ; echo -center '[ ANY KEY TO RETURN TO MENU ]'
read -key $keyname

return

# # # # # # # # # # # # # # #

callpoint: mkdir

# Make the specified directory.

# # # # # # # # # # # # # # #

if $arg:1 is file
   erase $arg:1
endif

if $arg:1 is not directory
   mkdir $arg:1
endif

# # # # # # # # # # # # # # #

callpoint: exception

# Handle a problem with the installation.

# # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during installation"
set $sys:errloc ? "( From: install.s )"

move -down; echo -scr "ERROR: $sys:errtxt $sys:errloc"
move -down; echo -scr "NOTE - Cannot install software..."
move -down; echo -scr "See 'README.TXT' for manual installation notes."
move -down; echo -scr "Wait ..."

move -home
sleep 3000
call -goto exit

# # # # # # # # # # # # # # #

callpoint: exit

# Exit the program.

# # # # # # # # # # # # # # #

return

# # # # # # # # # # # # # # #

callpoint: display_logo

# Display the NLS logo.
# Usage: call display_logo (xcoord) (ycoord)

# # # # # # # # # # # # # # #

# Setup variables as follows:

set -view $arg
set $xorigin = $1 # First argument.
set $yorigin = $2 # Second argument.

# Clear the logo region.

box -clear [ $xorigin - 2] [ $yorigin - 1] 40 3

# Draw block letters.

screen -back black
screen -fore red
box -style roman
box [ $xorigin      ] $yorigin 2 2;
box [ $xorigin + 4  ] $yorigin 2 2;
box [ $xorigin + 8  ] $yorigin 2 2;

# The NLS logo text is as follows:

move [ $xorigin + 1] [ $yorigin + 1 ]; echo -screen "N"
move [ $xorigin + 5] [ $yorigin + 1 ]; echo -screen "L"
move [ $xorigin + 9] [ $yorigin + 1 ]; echo -screen "S"

# Invert each block letter, for monochrome displays.

box -invert [ $xorigin      ] $yorigin 2 2;
box -invert [ $xorigin + 4  ] $yorigin 2 2;
box -invert [ $xorigin + 8  ] $yorigin 2 2;

# Address information.

screen -back black
screen -fore cyan
move [ $xorigin + 14 ] $yorigin
echo -screen "NEW LINE SOFTWARE, Inc."
move -down; echo -screen "P.O. Box 3433"
move -down; echo -screen "Englewood, CO 80155"

# Logo is being displayed. Return.

return
