
*  LOWLEVEL.PRG
*  Demonstrates some of FORCE's indirect addressing and low level
*  functionality.
*  Code by H. S. Curlee
*  Software Evolution Corp.
*  1994
******************************************************************
/*
this example shows how to use some of the low-level
Force functions to manipulate memory and screen attributes
First, we're going to declare and initialize a 13-element character array.

Then we'll clear the screen, use some of Force's low-level abilities to
determine the segment and offset of the array in memory.  After that we'll
use these values in a loop to grab & display the values in memory (the array
elements) to the screen.
Finally, we'll determine the start of the video buffer in memory and
write a new value directly to all the attribute bytes in the screen area
of video memory, thereby changing the screen color.
*/



#include screen.hdr
#include io.hdr
#include system.hdr
#include string.hdr



vardef     extern
byte      __color_std
enddef


VARDEF
char(8)     typesel[13]  ="PEEKTEST","AUTO 2DR","AUTO 3DR","AUTO 4DR","AUTO 5DR","CONVRTBL","P/UP 4X2","P/UP 4X4","UTIL 4X2","UTIL 4X4","MINIVAN ","VAN 4X2 ","VAN 4X4 "
ENDDEF


*--------------------------------------------
procedure force_main

vardef
uint       iCounter
int        iCycleCounter
enddef

clear

@ 0,2 ?? "We will now initialize a char array in memory when the file executes"
delay(2)
@ 1,2 ?? "The array definition and values are as follows"
@ 2,2 ?? "char(8)     typesel[13]  ='PEEKTEST','AUTO 2DR','AUTO 3DR','AUTO 4DR',;"
@ 3,2 ?? "'AUTO 5DR','CONVRTBL','P/UP 4X2','P/UP 4X4','UTIL 4X2','UTIL 4X4',;"
@ 4,2 ?? "'MINIVAN ','VAN 4X2 ','VAN 4X4 ' "
delay(3)
@ 6,2 ?? "Now we'll use Peek(),SegmentValue(), and OffsetValue() to read from memory"
delay(3)



iCycleCounter := 0

do while iCycleCounter<2

iCounter := 0

@ 8,0

                                   //delay 2 ticks between chars so user can
do while .T.                       //better visualize procedure progress

  ?? chr(peek(segmentValue(typesel[]), ;
         offsetvalue(typesel[])+iCounter))
   if iCycleCounter==0
   ticks(1)
   endif
              if peek(segmentValue(typesel[]), ;
                     offsetvalue(typesel[])+iCounter)==0

              ? ""       // start a new line for the next element if NULL
                         // (0 terminates each array element)
                 if iCycleCounter==0
                 delay(1)   // wait 1 sec between elements
                 endif
               endif
               if iCounter==116
               exit   // at end of array (13 elements*(8 chars+1 terminating0)
               endif
  iCounter++            // increment iCounter to hit next byte in memory
enddo                   // next time around


iCycleCounter++

               if iCycleCounter==2
               exit
               endif

BOX(7,40,22,76,"",__color_std)


@ 08,42 ?? "These characters were obtained"
@ 09,42 ?? "directly from memory via FORCE's"
@ 10,42 ?? "indirect addressing and lowlevel"
@ 11,42 ?? "capabilities."
delay(4)
@ 12,42 ?? "As you noticed, each character"
@ 13,42 ?? "was actually obtained and sent to"
@ 14,42 ?? "the screen individually.  A delay"
@ 15,42 ?? "was inserted to allow the user to"
@ 16,42 ?? "better visualize the process."
delay(4)
@ 17,42 ?? "Without the delay, it is virtually"
@ 18,42 ?? "instantaneous."
delay(3)
@ 19,42 ?? "Try that with Clipper or FoxPro!"
delay(3)
@ 20,42 ?? "Now, isn't this easier than C?"
delay(3)
@ 21,42 ?? "And all in 9800 bytes too!!!!"
delay(3)

@ 7,0 clear to 24,79


@ 23,20   ?? "WHAT!  You don't believe it's that fast?"
delay(2)
@ 24,20   ?? "OK, here we go again....direct from memory:"



  for iCounter=9 downto 1
  @ 24, 63 ?? iCounter:1
  delay(1)
  next
  @ 24, 63 ?? "0":1


enddo

@ 23,0 clear to 24,79


delay(2)

BOX(7,32,24,70,"ķӺ",__color_std)


@ 08,34 ?? "Now let's look at some of the other"
@ 09,34 ?? "powerful lowlevel capabilities of"
@ 10,34 ?? "FORCE 2.4c...."
delay(2)
@ 12,34 ?? "Here's an idea:  let's use"
@ 13,34 ?? "ScrSeg() to get the segment address"
@ 14,34 ?? "of the video screen adapter buffer."
delay(3)
@ 16,34 ?? "Then we can use Poke() to directly"
@ 17,34 ?? "write a new value to each attribute"
@ 18,34 ?? "byte of the screen area in memory,"
@ 19,34 ?? "and change the color thereby."
delay(4)
@ 20,34 ?? "Ready?"
delay(2)



iCounter:=1                       // start at offset 1 (first attribute byte)

do while iCounter<4097            // well, there are only 4096 bytes in
poke(ScrSeg(),iCounter,0x1f)      //  a screen...note that we only POKE
iCounter+=2                       //  each alternate byte as we only want
enddo                             //  to change the color (attribute) and
                                  //  each screen character is made up of
                                  //  two bytes: a character byte and an
                                  //  attribute byte.

delay(2)
@ 22,34 ?? "The source for this program is in"
@ 23,34 ?? "the file lowlevel.prg"
delay(2)



quit

endpro


