
{ WRMODE.PAS - BGI256 write mode definitions }
{ Written by Michael Day as of 26 Dec 94 }

{Usage:    SetWriteMode(Selection+Function); }
{Example:  SetWriteMode(FloodFillMode+SeedFill); }

{Remember, use of the extended SetWriteMode commands with a standard}
{BGI driver (other than BGI256) will leave the line draw mode in an}
{indetermant state. You should always call SetWriteMode just before}
{using a command that uses the line drawing functions to insure the state}
{of the line draw mode, or SetWriteMode immediately after using an extended}
{function to restore the last drawn mode. The Borland graph driver}
{does not provide information about the last selected line draw mode.}
{You will have to track that information yourself.}

{$R-}

unit WRMODE;
interface

{sprite record used with PutImage sprite command ($A0)}
type  SpriteRecord = record
                       Width,Height,OldX,OldY:integer;
                       Image,Old:pointer;
                     end;

type  RGBpaletteRec  = record
                         Red,Green,Blue:byte;
                       end;
      RGBpaletteType = array[0..255] of RGBpaletteRec;


const BGIVersion : word = 0;
      VirtualBGI : boolean = false;
      DisplayImagePages : word = 1;

      ScreenOfsX    : word = 0; {screen window position on virtual display}
      ScreenOfsY    : word = 0;
      DrawingOfsX   : word = 0; {drawing window position on virtual display}
      DrawingOfsY   : word = 0;

      ScreenWidth   : word = 320;
      ScreenHeight  : word = 200;
      DrawingWidth  : word = 320;
      DrawingHeight : word = 200;
      VirtualWidth  : word = 320;
      VirtualHeight : word = 200;
      ScanLength    : word = 320;

      {graphics mode selections available}
const Mode200  = 0;  {320x200x256}
      Mode400  = 1;  {640x400x256}
      Mode480  = 2;  {640x480x256}
      Mode600  = 3;  {800x600x256}
      Mode768  = 4;  {1024x768x256}
      Mode1024 = 5;  {1280x1024x256}
      AutoMode = 127;{AutoDetect mode}

      {SetWriteMode selection commands}
const LineMode       = $00;  {line drawing write style}
      PixelMode      = $20;  {pixel drawing write style}
      FillMode       = $40;  {fill write style}
      FloodFillType  = $60;  {floodfill option selection}
      TextMode       = $80;  {bitmapped text write style}
      GetImageMode   = $C0;  {GetImage write style}

      MiscCommand    = $E0;  {misc BGI driver commands}

      PutAniMode     = $A0;  {PutImage animation modifier cmd}

      {Write mode functions}
      MoveWrite    = 0;         {foreground and background drawing}
      XorWrite     = 1;
      OrWrite      = 2;
      AndWrite     = 3;
      NotMoveWrite = 4;
      NotXorWrite  = 5;
      NotOrWrite   = 6;
      NotAndWrite  = 7;
      ForeMoveWrite    = 8;     {foreground only drawing}
      ForeXorWrite     = 9;
      ForeOrWrite      = 10;
      ForeAndWrite     = 11;
      ForeNotMoveWrite = 12;
      ForeNotXorWrite  = 13;
      ForeNotOrWrite   = 14;
      ForeNotAndWrite  = 15;
      BackMoveWrite     = 16;   {background only drawing}
      BackXorWrite      = 17;
      BackOrWrite       = 18;
      BackAndWrite      = 19;
      BackNotMoveWrite  = 20;
      BackNotXorWrite   = 21;
      BackNotOrWrite    = 22;
      BackNotAndWrite   = 23;
      SetBackColor    = 24; {set the background color for draw method}
      GetWriteMode    = 30; {return selected write mode on GetMaxMode call}
      GetBackColor    = 31; {return background color on GetMaxMode call}

      {FloodFillType functions}
      BorderFill      = 0;  {use border floodfill method - default}
      SeedFill        = 1;  {use seed floodfill method}
      AutoFill        = 8;  {auto-select simplex or complex fill - default}
      ComplexFill     = 9;  {force complex floodfill always}
      FillCompressOff = 10; {use standard floodfill stack - default}
      FillCompressOn  = 11; {use compressed floodfill stack}
      FillDelayOff    = 12; {fill area while searching - default}
      FillDelayOn     = 13; {delay fill to after search, not active in simplex}
      FillTracerOff   = 14; {no tracing of search path - default}
      FillTracerOn    = 15; {show search path as it is processed}
      SetClearColorCmd = 24; {set initcolor (used by ClearDevice)}
      GetXYStackPeak   = 26; {ret last peak val of fill XYstack}
      GetXYStackFree   = 27; {ret last val of free XYstack space}
      GetFloodFillOpt  = 31; {ret FF option flags in next GetMaxMode call}

      {Misc Command functions}
      SetGetPixelReadOnly  = 0;  {set GetPixel to read monly mode}
      SetGetPixelReadWrite = 1;  {set GetPixel to write after read mode}
      SetGetImageReadOnly  = 2;  {set GetImage to read only mode}
      SetGetImageReadWrite = 3;  {set GetImage to write after read mode}
      SetLineStyleMode0    = 4;  {set mode 0 - (default) clr pat cnt on draw - standard}
      SetLineStyleMode1    = 5;  {set mode 1 - do not clr pat cnt on draw - continuous}
      SetLineStyleMode2    = 6;  {set mode 2 - fixed line pattern mask - fixed}
      SetPutImgBackColor   = 24; {set new putimage background color}

      {These misc cmd functions modify return value ofnext GetMaxMode call}
      GetCurrentModeCmd     = 25; {ret actual graph mode in use}
      GetDriverVersionCmd   = 28; {ret driver version number}
      GetPutImgWriteModeCmd = 30; {ret last write mode used by PutImage}
      GetPutImgBackColorCmd = 31; {ret background color used by PutImage}


      procedure SetScreenOfs(OfsX,OfsY:word);
      procedure SetDrawingOfs(OfsX,OfsY:word);
      procedure SetDrawingSize(Width,Height:word);
      procedure SetVirtualWidth(Width:word);

      procedure GetScreenOfs;   {update screen X/Y offset in virtual display}
      procedure GetDrawingOfs;  {update drawing X/Y offset in virtual display}
      procedure GetScreenSize;  {update screen size}
      procedure GetDrawingSize; {update drawing area size}
      procedure GetVirtualSize; {update virtual display size}

      procedure UpdateDisplayInfo;   {update full virtual screen info}
      procedure UpdateScreenInfo;    {update drawing and screen info only}
      procedure GetBGIVersion;       {get BGI256 version info}
      procedure GetVirtualFlag;      {get current virtual state 1=virtualBGI}
      procedure GetScanLineLength;   {update scan line length info}
      procedure GetRGBPalette(Which:word; var Red,Green,Blue:byte);
      procedure SetFullRGBPalette(Start,Count:word; var Pal);
      procedure SetClearColor(Color:word);

      function DisplaySync:longint;  {wait for vert interrupt}
      function VideoMemorySize:word; {ret vid mem size in KB,   0=unknown}
      function GetImagePages:word;   {get number of image pages 0=unknown}


implementation
uses graph;

var   tCPX,tCPY : integer;

const DisplayCommand = $A0;  {misc display commands}

      {Misc Display functions}
const SetScreenOffsetCmd  = 0; {Set screen window X offset to cpx/cpy val}
      SetDrawingOffsetCmd = 1; {Set drawing window offset to cpx/cpy val}
      SetDrawingSizeCmd   = 2; {Set drawing window size to cpx/cpy val}
      SetVirtualSizeCmd   = 3; {Set virtual screen size to cpx/cpy val}

      GetScreenOffsetXCmd   = 4; {ret screen window X offset}
      GetScreenOffsetYCmd   = 5; {ret screen window Y offset}
      GetDrawingOffsetXCmd  = 6; {ret drawing window X offset}
      GetDrawingOffsetYCmd  = 7; {ret drawing window Y offset}

      GetScreenWidthCmd     = 8; {ret phys scrn width}
      GetScreenHeightCmd    = 9; {ret phys scrn height}
      GetDrawingWidthCmd    = 10; {ret drawing window width}
      GetDrawingHeightCmd   = 11; {ret drawing window height}
      GetVirtualWidthCmd    = 12; {ret virtual window width}
      GetVirtualHeightCmd   = 13; {ret virtual window height}

      GetScanLineLengthCmd  = 28; {ret scan line len}
      GetVirtualStatusCmd   = 29; {ret virtual display status 0=no, 1=yes}
      GetVidMemSizeCmd      = 30; {ret video memory size in kilobytes}
      GetMaxImagePagesCmd   = 31; {ret max number image pages possible}


{-------------------------------------------------}
{misc stuff}

{wait for the start of vertical sync}
function DisplaySync:longint;
var aborttime:word;
begin
  aborttime := 1;
  while ((port[$3DA] and 8) = 8) do
  begin
    inc(aborttime);
    if aborttime > $1fffff then
    begin
      aborttime := 0;
      Exit;
    end;
  end;
  while ((port[$3DA] and 8) = 0) and (aborttime < $1fffff) do
  begin
    inc(aborttime);
    if aborttime > $1fffff then
    begin
      aborttime := 0;
      Exit;
    end;
  end;
  DisplaySync := aborttime;
end;


{-------------------------------------------------}
{get BGI256 version. 0=not me, 200=old V2.00 driver, 300=old V3.00 driver}
{                    2.03 or 3.03 and up= newer driver versions}
procedure GetBGIVersion;
var Result : integer;
begin
  BGIVersion := 0;
  VirtualBGI := false;
  Result := GraphResult;  {clear result flag}
  if GetMaxColor < 255 then Exit; {not 256 color, must be a Borland driver}
  Result := GraphResult;
  if Result <> 0 then Exit;
  SetWriteMode(MiscCommand+GetDriverVersionCmd);
  Result := GetMaxMode;
  SetWriteMode(0);
  if ((Result >= 203) and (Result <= 299)) or
     ((Result >= 303) and (Result <= 9999)) then
  begin
    BGIVersion := Result; {Yahoo! It's us!}
    SetWriteMode(DisplayCommand+GetVirtualStatusCmd);
    Result := GetMaxMode;
    if Result = 1 then
      VirtualBGI := true; {yeah! Virtual screen is supported!}
    Exit;
  end;

  SetColor(255);  {try to read background color to see if it is BGI256 V3}
  SetWriteMode(MiscCommand+LineMode+SetBackColor);
  SetWriteMode(MiscCommand+LineMode+GetBackColor);
  Result := GetMaxMode;
  SetWriteMode(0);
  if Result = 255 then
  begin
    BGIVersion := 302;  {in version 3.02 we could read the backgroundcolor}
    Exit;
  end;
  {at this point I don't know what it is, it could be V2.02 or V2.06 BGI256}
  {or some other 3rd party BGI driver}
end;

{-------------------------------------------------}
{display procedures}

procedure SetScreenOfs(OfsX,OfsY:word);
begin
  if VirtualBGI then
  begin
    tCPX := GetX;
    tCPY := GetY;
    moveto(OfsX,OfsY);
    SetWriteMode(DisplayCommand+SetScreenOffsetCmd);
    moveto(tCPX,tCPY);
  end;
  GetScreenOfs;
end;

procedure SetDrawingOfs(OfsX,OfsY:word);
begin
  if VirtualBGI then
  begin
    tCPX := GetX;
    tCPY := GetY;
    moveto(OfsX,OfsY);
    SetWriteMode(DisplayCommand+SetDrawingOffsetCmd);
    moveto(tCPX,tCPY);
  end;
  GetDrawingOfs;
end;

{remember to call: SetGraphMode(GetGraphMode); after this procedure}
{followed by UpdateScreenInfo or UpdateDisplayInfo if VirtualWidth changed.}
{If both SetDrawingSize and SetVirtualWidth are called, this only needs}
{to be done once after the calls.}
procedure SetDrawingSize(Width,Height:word);
begin
  if VirtualBGI then
  begin
    tCPX := GetX;
    tCPY := GetY;
    moveto(Width,Height);
    SetWriteMode(DisplayCommand+SetDrawingSizeCmd);
    moveto(tCPX,tCPY);
  end;
  GetDrawingSize;
end;

{remember to call: SetGraphMode(GetGraphMode); after this procedure}
{followed by UpdateDisplayInfo}
procedure SetVirtualWidth(Width:word);
begin
  if VirtualBGI then
  begin
    tCPX := GetX;
    tCPY := GetY;
    moveto(Width,0);  {Y parameter is not used}
    SetWriteMode(DisplayCommand+SetVirtualSizeCmd);
    moveto(tCPX,tCPY);
  end;
  GetVirtualSize;
end;


procedure GetVirtualSize;
begin
  if VirtualBGI then
  begin
    SetWriteMode(DisplayCommand+GetVirtualWidthCmd);
    VirtualWidth := GetMaxMode;
    SetWriteMode(DisplayCommand+GetVirtualHeightCmd);
    VirtualHeight := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    VirtualWidth  := succ(GetMaxX);
    VirtualHeight := succ(GetMaxY);
  end;
end;

procedure GetDrawingSize;
begin
  if VirtualBGI then
  begin
    SetWriteMode(DisplayCommand+GetDrawingWidthCmd);
    DrawingWidth := GetMaxMode;
    SetWriteMode(DisplayCommand+GetDrawingHeightCmd);
    DrawingHeight := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    DrawingWidth  := succ(GetMaxX);
    DrawingHeight := succ(GetMaxY);
  end;
end;

procedure GetDrawingOfs;
begin
  if VirtualBGI then
  begin
    SetWriteMode(DisplayCommand+GetDrawingOffsetXCmd);
    DrawingOfsX := GetMaxMode;
    SetWriteMode(DisplayCommand+GetDrawingOffsetYCmd);
    DrawingOfsY := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    DrawingOfsX := 0;
    DrawingOfsY := 0;
  end;
end;

procedure GetScreenOfs;
begin
  if VirtualBGI then
  begin
    SetWriteMode(DisplayCommand+GetScreenOffsetXCmd);
    ScreenOfsX := GetMaxMode;
    SetWriteMode(DisplayCommand+GetScreenOffsetYCmd);
    ScreenOfsY := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    ScreenOfsX := 0;
    ScreenOfsY := 0;
  end;
end;

procedure GetScreenSize;
begin
  if VirtualBGI then
  begin
    SetWriteMode(DisplayCommand+GetScreenWidthCmd);
    ScreenWidth := GetMaxMode;
    SetWriteMode(DisplayCommand+GetScreenHeightCmd);
    ScreenHeight := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    ScreenWidth  := succ(GetMaxX);
    ScreenHeight := succ(GetMaxY);
  end;
end;

procedure GetScanLineLength;
begin
  if VirtualBGI then
  begin
    SetWriteMode(DisplayCommand+GetScanLineLengthCmd);
    ScanLength := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    ScanLength := succ(GetMaxX);
  end;
end;

procedure GetVirtualFlag;
begin
  if (BGIversion > 0) and (BGIversion <> 302) then
  begin
     SetWriteMode(DisplayCommand+GetVirtualStatusCmd);
     if GetMaxMode = 1 then
       VirtualBGI := true
     else
       VirtualBGI := false;
     SetWriteMode(0);
  end
  else
  begin
    VirtualBGI := false;
  end;
end;


procedure UpdateScreenInfo;
begin
  GetDrawingSize;
  GetScreenSize;
  GetScreenOfs;
  GetDrawingOfs;
  GetImagePages;
end;

procedure UpdateDisplayInfo;
begin
  GetBGIversion;
  GetScanLineLength;
  GetVirtualSize;
  UpdateScreenInfo;
end;


{----------------------------------------------------}
{ misc display functions }

function VideoMemorySize:word;
begin
  if (BGIversion > 0) and (BGIversion <> 302) then
  begin
     SetWriteMode(DisplayCommand+GetVidMemSizeCmd);
     VideoMemorySize := GetMaxMode;
     SetWriteMode(0);
  end
  else
  begin
    VideoMemorySize := 0;  {zero = unknown}
  end;
end;

function GetImagePages:word;
begin
  if (BGIversion > 0) and (BGIversion <> 302) then
  begin
    SetWriteMode(DisplayCommand+GetMaxImagePagesCmd);
    DisplayImagePages := GetMaxMode;
    SetWriteMode(0);
  end
  else
  begin
    GetImagePages := 1;  {just ret one if unknown}
  end;
end;


{----------------------------------------------------}
{Palette functions}

procedure GetRGBPalette(Which:word; var Red,Green,Blue:byte);
var rdh,rch,rcl:byte;
begin
  asm
    MOV  AX,$1015
    MOV  BX,[Which]
    INT  $10
    MOV  [rdh],DH
    MOV  [rch],CH
    MOV  [rcl],CL
  end;
  Red := rdh;
  Green := rch;
  Blue := rcl;
end;

procedure SetFullRGBPalette(Start,Count:word; var Pal); assembler;
{var PalArray : RGBpaletteType absolute Pal;}
asm
  MOV  AX,$1012
  MOV  BX,[Start]
  MOV  CX,[Count]
  LES  DX,[Pal]
  INT  $10
end;

procedure SetClearColor(Color:word);
var Temp : word;
begin
  Temp := GetColor;
  SetColor(Color);
  SetWriteMode(FloodFillType+SetClearColorCmd);
  SetColor(Temp);
end;


end.
